'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deleteEpic = exports.postEpic = undefined;

var _apiActionCreators = require('../sync/apiActionCreators');

var api = _interopRequireWildcard(_apiActionCreators);

require('rxjs/add/operator/do');

require('rxjs/add/operator/map');

require('rxjs/add/operator/filter');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/**
 * Mutate epic  provides a generic interface for triggering POST and DELETE requests
 * and dispatching particular actions with the API response. The POST or DELETE action
 * must follow this structure:
 *
 * ```
 * {
 *   type: 'POST_<SOMETHING>' (or '<SOMETHING>_POST'),
 *   payload: {
 *     query: a 'reactive query' object that maps to an API call,
 *     onSuccess: a callback that takes the API response as an argument, and
 *       returns an action object. The middleware takes care of dispatch
 *     onError: a callback that takes an Error argument and returns an action
 *       object
 *   }
 * }
 * ```
 *
 * This structure usually allows the success/error handling code to be bundled
 * alongside the POST/DELETE action creator, with the expectation that all response
 * processing can be done there
 *
 * @deprecated
 * @module MutateEpic
 */

/*
 * Delegate method-specific request actions (e.g. 'POST_MY_FORM') to the sync
 * middleware by modifying the request actions' query payloads to include
 * request method properties
 * @deprecated
 */
var _getMethodEpic = function _getMethodEpic(method) {
  return function (action$) {
    return action$.filter(function (_ref) {
      var type = _ref.type;
      return type.endsWith('_' + method.toUpperCase()) || type.startsWith(method.toUpperCase() + '_');
    }).do(function (_ref2) {
      var type = _ref2.type;

      // Webpack will make sure process.env.NODE_ENV is populated on client
      if (process.env.NODE_ENV !== 'production') {
        // using a dev endpoint, render a deprecation warning
        console.warn('This application is using Post/Delete middleware through ' + type + '.\nSee the platform Queries Recipes docs for refactoring options:\nhttps://github.com/meetup/meetup-web-platform/blob/master/docs/Queries.md#recipes');
      }
    }).map(function (_ref3) {
      var _ref3$payload = _ref3.payload,
          query = _ref3$payload.query,
          onSuccess = _ref3$payload.onSuccess,
          onError = _ref3$payload.onError;
      return {
        query: query,
        meta: { onSuccess: onSuccess, onError: onError }
      };
    }).map(function (_ref4) {
      var query = _ref4.query,
          meta = _ref4.meta;

      var actionCreator = method === 'delete' ? 'del' : method;
      return api[actionCreator](query, meta);
    });
  };
};

var postEpic = exports.postEpic = _getMethodEpic('post');
var deleteEpic = exports.deleteEpic = _getMethodEpic('delete');
//# sourceMappingURL=index.js.map