'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.responseToState = exports.DEFAULT_APP_STATE = exports.DEFAULT_API_STATE = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.api = api;
exports.app = app;

var _apiActionCreators = require('./sync/apiActionCreators');

var _cacheActionCreators = require('./cache/cacheActionCreators');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var DEFAULT_API_STATE = exports.DEFAULT_API_STATE = { inFlight: [] };
var DEFAULT_APP_STATE = exports.DEFAULT_APP_STATE = {};

var filterKeys = function filterKeys(obj, keys) {
	return Object.keys(obj).reduce(function (newObj, key) {
		if (!keys.includes(key)) {
			newObj[key] = obj[key];
		}
		return newObj;
	}, _extends({}, DEFAULT_API_STATE));
};

var responseToState = exports.responseToState = function responseToState(response) {
	return _defineProperty({}, response.ref, response);
};

/*
 * The primary reducer for data provided by the API
 */
function api() {
	var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_API_STATE;
	var action = arguments[1];

	switch (action.type) {
		case _apiActionCreators.API_REQ:
			{
				var requestRefs = (action.payload || []).map(function (_ref2) {
					var ref = _ref2.ref;
					return ref;
				});
				var inFlight = state.inFlight.filter(function (ref) {
					return !requestRefs.includes[ref];
				}) // clean out current duplicates
				.concat(requestRefs); // add new requested refs

				if ((action.meta || {}).logout) {
					// clear app state during logout
					return _extends({}, DEFAULT_API_STATE, { inFlight: inFlight });
				}

				// remove any `ref`s that are being refreshed - eliminate stale data
				var newState = _extends({}, filterKeys(state, requestRefs), {
					inFlight: inFlight
				});

				return newState;
			}
		case _apiActionCreators.API_RESP_SUCCESS: // fall though
		case _apiActionCreators.API_RESP_ERROR:
		case _cacheActionCreators.CACHE_SUCCESS:
			// fall through
			// each of these actions provides an API response that should go into app
			// state - error responses will contain error info
			delete state.fail; // if there are any values, the API is not failing
			return _extends({}, state, responseToState((action.payload || {}).response));
		case _apiActionCreators.API_RESP_FAIL:
			return _extends({}, state, { fail: action.payload });
		case _apiActionCreators.API_RESP_COMPLETE:
			{
				// always called - clean up inFlight
				var refs = (action.payload || []).map(function (_ref3) {
					var ref = _ref3.ref;
					return ref;
				});
				var _inFlight = state.inFlight.filter(function (ref) {
					return !refs.includes(ref);
				});
				return _extends({}, state, {
					inFlight: _inFlight
				});
			}
		default:
			return state;
	}
}

/**
 * The old API results store
 * @deprecated
 */
function app() {
	var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_APP_STATE;
	var action = arguments[1];

	var newState = void 0;

	switch (action.type) {
		case 'API_REQUEST':
			if ((action.meta || {}).logout) {
				return DEFAULT_APP_STATE; // clear app state during logout
			}
			return state;
		case 'API_SUCCESS':
			// API_SUCCESS contains an array of responses, but we just need to build a single
			// object to update state with
			newState = action.payload.responses.reduce(function (s, r) {
				return _extends({}, s, r);
			}, {});
			delete state.error;
			return _extends({}, state, newState);
		case 'API_ERROR':
			return _extends({}, state, {
				error: action.payload
			});
		default:
			return state;
	}
}
//# sourceMappingURL=reducer.js.map