'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getFetchQueriesEpic = exports.apiRequestToApiReq = exports.locationSyncEpic = exports.getNavEpic = undefined;
exports.getDeprecatedSuccessPayload = getDeprecatedSuccessPayload;
exports.default = getSyncEpic;

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/empty');

require('rxjs/add/observable/from');

require('rxjs/add/observable/fromPromise');

require('rxjs/add/observable/merge');

require('rxjs/add/observable/of');

require('rxjs/add/operator/catch');

require('rxjs/add/operator/map');

require('rxjs/add/operator/mergeMap');

require('rxjs/add/operator/takeUntil');

var _reduxObservable = require('redux-observable');

var _clickState = require('../../plugins/tracking/util/clickState');

var _router = require('../../router');

var _util = require('../../router/util');

var _apiActionCreators = require('./apiActionCreators');

var api = _interopRequireWildcard(_apiActionCreators);

var _syncActionCreators = require('./syncActionCreators');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } // mwp-tracking-plugin/util/clickState
// mwp-router
// mwp-router/util

/**
 * @deprecated
 */
function getDeprecatedSuccessPayload(successes, errors) {
	var allQueryResponses = [].concat(_toConsumableArray(successes), _toConsumableArray(errors));
	return allQueryResponses.reduce(function (payload, _ref) {
		var query = _ref.query,
		    response = _ref.response;

		if (!response) {
			return payload;
		}

		var ref = response.ref,
		    error = response.error,
		    responseBody = _objectWithoutProperties(response, ['ref', 'error']);

		if (error) {
			// old payload expects error as a property of `value`
			responseBody.value = { error: error };
		}
		payload.queries.push(query);
		payload.responses.push(_defineProperty({}, ref, responseBody));
		return payload;
	}, { queries: [], responses: [] });
}

/**
 * Any operations that keep the browser application in sync with the
 * server should be implemented here.
 *
 * - Navigation-generated API request handling
 * - Arbitrary API request handling (syncActionCreators.apiSuccess)
 *
 * @module syncEpic
 */

/**
 * Navigation actions will provide the `location` as the payload, which this
 * epic will use to collect the current Reactive Queries associated with the
 * active routes.
 *
 * These queries will then be dispatched in the payload of `apiRequest`. Any
 * metadata about the navigation action can also be sent to the `apiRequest`
 * here.
 *
 * @param {Object} routes The application's React Router routes
 * @returns {Function} an Epic function that emits an API_REQUEST action
 */
var getNavEpic = exports.getNavEpic = function getNavEpic(routes, baseUrl) {
	var resolveRoutes = (0, _util.getRouteResolver)(routes, baseUrl);
	return function (action$, store) {
		return action$.ofType(_router.LOCATION_CHANGE, _router.SERVER_RENDER).mergeMap(function (_ref2) {
			var location = _ref2.payload;

			// note that this function executes _downstream_ of reducers, so the
			// new `routing` data has already been populated in `state`
			var state = store.getState();
			// inject request metadata from context, including `store.getState()`
			var requestMetadata = {
				referrer: (state.routing.referrer || {}).pathname || '',
				logout: location.pathname.endsWith('logout'), // assume logout route ends with logout
				clickTracking: state.clickTracking
			};

			var cacheAction$ = requestMetadata.logout ? _Observable.Observable.of({ type: 'CACHE_CLEAR' }) : _Observable.Observable.empty();

			var apiAction$ = _Observable.Observable.fromPromise(resolveRoutes(location, baseUrl).then((0, _util.getMatchedQueries)(location))).map(function (q) {
				return api.requestAll(q, requestMetadata);
			});

			var clickAction$ = _Observable.Observable.of(_clickState.actions.clear());

			return _Observable.Observable.merge(cacheAction$, apiAction$, clickAction$);
		});
	};
};

/**
 * Any action that should reload the API data should be handled here, e.g.
 * LOGIN_SUCCESS, which should force the app to reload in an 'authorized'
 * state
 *
 * Note: this action is only possible in the browser, not the server, so
 * `browserHistory` is safe to use here.
 */
var locationSyncEpic = exports.locationSyncEpic = function locationSyncEpic(action$, store) {
	return action$.ofType('LOGIN_SUCCESS').ignoreElements() // TODO: push window.location into history without querystring
	.map(function () {
		return { type: _router.LOCATION_CHANGE, payload: window.location };
	});
};

/**
 * Old apiRequest maps directly onto new api.requestAll
 * @deprecated
 */
var apiRequestToApiReq = exports.apiRequestToApiReq = function apiRequestToApiReq(action$) {
	return action$.ofType('API_REQUEST').map(function (action) {
		return api.requestAll(action.payload, action.meta);
	});
};

/**
 * Listen for API_REQ and generate response actions from fetch results
 *
 * emits
 * - 1 or more API_RESP_SUCCESS
 * - 1 or more API_RESP_ERROR
 * - API_SUCCESS  // deprecated
 * - API_COMPLETE
 *
 * or
 *
 * - API_RESP_FAIL
 * - API_ERROR  // deprecated
 * - API_COMPLETE
 */
var getFetchQueriesEpic = exports.getFetchQueriesEpic = function getFetchQueriesEpic(fetchQueriesFn) {
	return function (action$, store) {
		return action$.ofType(api.API_REQ).mergeMap(function (_ref3) {
			var queries = _ref3.payload,
			    meta = _ref3.meta;

			// set up the fetch call to the app server
			var _store$getState = store.getState(),
			    config = _store$getState.config;

			var fetchQueries = fetchQueriesFn(config.apiUrl);
			return _Observable.Observable.fromPromise(fetchQueries(queries, meta)) // call fetch
			.takeUntil(action$.ofType(_router.LOCATION_CHANGE)) // cancel this fetch when nav happens
			.mergeMap(function (_ref4) {
				var _ref4$successes = _ref4.successes,
				    successes = _ref4$successes === undefined ? [] : _ref4$successes,
				    _ref4$errors = _ref4.errors,
				    errors = _ref4$errors === undefined ? [] : _ref4$errors;

				// meta contains a Promise that must be resolved
				meta.resolve([].concat(_toConsumableArray(successes), _toConsumableArray(errors)));
				var deprecatedSuccessPayload = getDeprecatedSuccessPayload(successes, errors);
				var deprecatedActions = [(0, _syncActionCreators.apiSuccess)(deprecatedSuccessPayload)];
				if (meta && meta.onSuccess) {
					deprecatedActions.push(meta.onSuccess(deprecatedSuccessPayload));
				}
				var actions = [].concat(_toConsumableArray(successes.map(api.success)), _toConsumableArray(errors.map(api.error)), deprecatedActions, [api.complete(queries)]);
				return _Observable.Observable.of.apply(_Observable.Observable, _toConsumableArray(actions));
			}).catch(function (err) {
				// meta contains a Promise that must be rejected
				meta.reject(err);
				var deprecatedActions = [(0, _syncActionCreators.apiError)(err)];
				if (meta && meta.onError) {
					deprecatedActions.push(meta.onError(err));
				}
				return _Observable.Observable.of.apply(_Observable.Observable, [api.fail(err)].concat(deprecatedActions, [api.complete(queries)]));
			});
		});
	};
};

function getSyncEpic(routes, fetchQueries, baseUrl) {
	return (0, _reduxObservable.combineEpics)(getNavEpic(routes, baseUrl),
	// locationSyncEpic,
	getFetchQueriesEpic(fetchQueries), apiRequestToApiReq // TODO: remove in v3 - apiRequest is deprecated
	);
}
//# sourceMappingURL=index.js.map