'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = start;

require('../util/globals');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _spdy = require('spdy');

var _spdy2 = _interopRequireDefault(_spdy);

var _config = require('../util/config');

var _config2 = _interopRequireDefault(_config);

var _getPlugins = require('./util/getPlugins');

var _getPlugins2 = _interopRequireDefault(_getPlugins);

var _util = require('./util');

var _routes = require('./routes');

var _routes2 = _interopRequireDefault(_routes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } // eventually this will be a native node module

/**
 * @module server
 */

/**
 * The start function applies the rendering function to the correct application
 * route and combines the provided routes and plugins with the base routes
 * and plugins
 *
 * @param {Object} languageRenderers A mapping of localeCodes to functions that emit
 *   the rendered HTML for the locale-specific request
 * @param {Array} routes additional routes for the app - cannot include a
 *   wildcard route
 * @param {Array} plugins additional plugins for the server, usually to support
 *   features in the additional routes
 * @return {Promise} the Promise returned by Hapi's `server.connection` method
 */
function start(languageRenderers, _ref) {
	var _ref$routes = _ref.routes,
	    routes = _ref$routes === undefined ? [] : _ref$routes,
	    _ref$plugins = _ref.plugins,
	    plugins = _ref$plugins === undefined ? [] : _ref$plugins;

	// source maps make for better stack traces
	// we might not want this in production if it makes anything slower
	require('source-map-support').install();

	(0, _util.configureEnv)(_config2.default);

	var baseRoutes = (0, _routes2.default)();
	var finalRoutes = [].concat(_toConsumableArray(routes), _toConsumableArray(baseRoutes));

	var connection = {
		host: '0.0.0.0',
		port: _config2.default.app_server.port,
		routes: {
			plugins: {
				'electrode-csrf-jwt': {
					enabled: false
				}
			}
		}
	};

	if (_config2.default.app_server.protocol === 'https') {
		// enable HTTP/2
		connection.tls = true;
		connection.listener = _spdy2.default.createServer({
			key: _fs2.default.readFileSync(_config2.default.app_server.key_file),
			cert: _fs2.default.readFileSync(_config2.default.app_server.crt_file)
		});
	}

	var finalPlugins = [].concat(_toConsumableArray(plugins), _toConsumableArray((0, _getPlugins2.default)({ languageRenderers: languageRenderers })));

	_config2.default.supportedLangs = Object.keys(languageRenderers);
	return (0, _util.server)(finalRoutes, connection, finalPlugins, _config2.default);
}
//# sourceMappingURL=index.js.map