'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.setCsrfCookies = setCsrfCookies;
exports.getCsrfPlugin = getCsrfPlugin;
exports.getAppRoutePlugin = getAppRoutePlugin;
exports.getRequestAuthPlugin = getRequestAuthPlugin;
exports.getLogger = getLogger;
exports.getActivityTrackingPlugin = getActivityTrackingPlugin;
exports.getClickTrackingPlugin = getClickTrackingPlugin;
exports.getApiProxyPlugin = getApiProxyPlugin;
exports.default = getPlugins;

var _inert = require('inert');

var _inert2 = _interopRequireDefault(_inert);

var _pino = require('pino');

var _pino2 = _interopRequireDefault(_pino);

var _hapiPino = require('hapi-pino');

var _hapiPino2 = _interopRequireDefault(_hapiPino);

var _electrodeCsrfJwt = require('electrode-csrf-jwt');

var _electrodeCsrfJwt2 = _interopRequireDefault(_electrodeCsrfJwt);

var _config = require('mwp-cli/src/config');

var _config2 = _interopRequireDefault(_config);

var _logger = require('../../util/logger');

var _logger2 = _interopRequireDefault(_logger);

var _appRoute = require('../../plugins/app-route');

var _appRoute2 = _interopRequireDefault(_appRoute);

var _requestAuthPlugin = require('../../plugins/requestAuthPlugin');

var _requestAuthPlugin2 = _interopRequireDefault(_requestAuthPlugin);

var _activity = require('../../plugins/tracking/activity');

var _activity2 = _interopRequireDefault(_activity);

var _click = require('../../plugins/tracking/click');

var _click2 = _interopRequireDefault(_click);

var _language = require('../../plugins/language');

var _language2 = _interopRequireDefault(_language);

var _serviceWorker = require('../../plugins/service-worker');

var _serviceWorker2 = _interopRequireDefault(_serviceWorker);

var _apiProxy = require('../../plugins/api-proxy');

var _apiProxy2 = _interopRequireDefault(_apiProxy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Hapi plugins for the dev server
 *
 * @module ServerPlugins
 */

function setCsrfCookies(request, reply) {
	var csrfHeader = (request.response.headers || {})['x-csrf-jwt'];
	if (csrfHeader) {
		reply.state('x-csrf-jwt-header', csrfHeader);
	}
	return reply.continue();
}

/**
 * The CSRF plugin we use - 'electrode-csrf-jwt' compares a cookie token to a
 * header token in non-GET requests. By default, it will set the cookie token
 * itself ('x-csrf-jwt'), and supply the corresponding header token in a custom
 * header (also 'x-csrf-jwt'). However, we update this flow to also supply the
 * header token as a cookie ('x-csrf-jwt-header') so that it syncs across
 * browser tabs.
 *
 * In order to ensure that both cookie values have parallel settings, this
 * function calls `server.state` for both cookie names before registering the
 * plugin.
 *
 * @return {Object} the { register } object for a `server.register` call.
 */
function getCsrfPlugin() {
	var register = function register(server, options, next) {
		var cookieOptions = {
			path: '/',
			isSecure: server.settings.app.isProd
		};

		options.secret = server.settings.app.csrf_secret;

		server.state('x-csrf-jwt', // set by plugin
		_extends({}, cookieOptions, { isHttpOnly: true // no client-side interaction needed
		}));

		server.state('x-csrf-jwt-header', // set by onPreResponse
		_extends({}, cookieOptions, { isHttpOnly: false // the client must read this cookie and return as a custom header
		}));

		var registration = _electrodeCsrfJwt2.default.register(server, options, next);
		server.ext('onPreResponse', setCsrfCookies); // this extension must be registered _after_ plugin is registered

		return registration;
	};

	register.attributes = _electrodeCsrfJwt2.default.register.attributes;

	return {
		register: register
	};
}

function getAppRoutePlugin(options) {
	return {
		register: _appRoute2.default,
		options: options
	};
}
/**
 * configure and return the plugin that
 * allows requests to get anonymous oauth tokens
 * to communicate with the API
 */
function getRequestAuthPlugin() {
	return {
		register: _requestAuthPlugin2.default
	};
}

function getLogger() {
	var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { logEvents: ['onPostStart', 'onPostStop', 'response'] };

	var onRequestError = function onRequestError(request, err) {
		console.error(JSON.stringify({
			err: err.stack,
			req: _pino2.default.stdSerializers.req(request.raw.req),
			res: _pino2.default.stdSerializers.res(request.raw.res),
			message: '500 Internal server error: ' + err.message
		}));
	};
	var register = function register(server, options, next) {
		server.on('request-error', onRequestError);
		return _hapiPino2.default.register(server, options, next);
	};
	register.attributes = _hapiPino2.default.register.attributes;

	options.instance = _logger2.default;
	return {
		register: register,
		options: options
	};
}

function getActivityTrackingPlugin(_ref) {
	var agent = _ref.agent,
	    isProd = _ref.isProd;

	return {
		register: _activity2.default,
		options: {
			agent: agent,
			isProd: isProd
		}
	};
}

function getClickTrackingPlugin() {
	return {
		register: _click2.default
	};
}

function getServiceWorkerPlugin() {
	return {
		register: _serviceWorker2.default
	};
}

function getApiProxyPlugin() {
	return {
		register: _apiProxy2.default
	};
}

function getLanguagePlugin() {
	return {
		register: _language2.default
	};
}

function getPlugins(_ref2) {
	var languageRenderers = _ref2.languageRenderers;
	var agent = _config2.default.package.agent,
	    isProd = _config2.default.env.properties.isProd;

	return [getAppRoutePlugin({ languageRenderers: languageRenderers }), getLogger(), getCsrfPlugin(), getRequestAuthPlugin(), getActivityTrackingPlugin({ agent: agent, isProd: isProd }), getClickTrackingPlugin(), getServiceWorkerPlugin(), getApiProxyPlugin(), getLanguagePlugin(), _inert2.default];
}
//# sourceMappingURL=getPlugins.js.map