'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.parseRequestQueries = parseRequestQueries;

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _rison = require('rison');

var _rison2 = _interopRequireDefault(_rison);

var _validation = require('./util/validation');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseRequestQueries(request) {
	var method = request.method,
	    mime = request.mime,
	    payload = request.payload,
	    query = request.query;

	var queriesRison = (method === 'post' || method === 'patch') && mime !== 'multipart/form-data' ? payload.queries : query.queries;

	if (!queriesRison) {
		return null;
	}

	var validatedQueries = _joi2.default.validate(_rison2.default.decode_array(queriesRison), _joi2.default.array().items(_validation.querySchema));
	if (validatedQueries.error) {
		throw validatedQueries.error;
	}
	return validatedQueries.value;
}

/*
 * The default API proxy handler - just calls the decorated `proxyApi$` method
 * and serializes the API responses
 */

exports.default = function (request, reply) {
	request.proxyApi$(parseRequestQueries(request)).subscribe(function (responses) {
		return reply(JSON.stringify({ responses: responses })).type('application/json');
	}, function (err) {
		return reply(err);
	} // 500 error - will only be thrown on bad implementation
	);
};
//# sourceMappingURL=handler.js.map