'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.setPluginState = exports.API_ROUTE_PATH = exports.API_PROXY_PLUGIN_NAME = undefined;
exports.default = register;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _duotone = require('./util/duotone');

var _routes = require('./routes');

var _routes2 = _interopRequireDefault(_routes);

var _proxy = require('./proxy');

var _proxy2 = _interopRequireDefault(_proxy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var API_PROXY_PLUGIN_NAME = exports.API_PROXY_PLUGIN_NAME = 'api-proxy';
var API_ROUTE_PATH = exports.API_ROUTE_PATH = '/mu_api';

/*
 * When response is sent, the plugin needs to delete any files that were
 * uploaded to the POST/PATCH endpoint
 */
var onResponse = function onResponse(request) {
	var uploads = request.plugins[API_PROXY_PLUGIN_NAME].uploads;
	var logger = request.server.app.logger;

	if (uploads.length) {
		var info = { info: uploads, req: request.raw.req };
		// $FlowFixMe - promisify not yet defined in flow-typed
		Promise.all(uploads.map(_util2.default.promisify(_fs2.default.unlink))).then(function () {
			logger.info(info, 'Deleted uploaded file(s)');
		}, function (err) {
			logger.error(info, 'Could not delete uploaded file(s)');
		});
	}
};

var setPluginState = exports.setPluginState = function setPluginState(request, reply) {
	request.plugins[API_PROXY_PLUGIN_NAME] = {
		setState: reply.state, // allow plugin to proxy cookies from API
		uploads: [] // keep track of any files that were uploaded
	};

	return reply.continue();
};

function register(server, options, next) {
	// supply duotone urls through `server.plugins['api-proxy'].duotoneUrls`
	server.expose('duotoneUrls', (0, _duotone.getDuotoneUrls)(_duotone.duotones, server.settings.app.photo_scaler_salt));

	// add a method to the `request` object that can call REST API
	server.decorate('request', 'proxyApi$', _proxy2.default, { apply: true });
	// plugin state must be available to all routes that use `request.proxyApi$`
	server.ext('onRequest', setPluginState);
	// clean up request state once response is sent
	server.on('response', onResponse);

	// add a route that will receive query requests as querystring params
	var routes = (0, _routes2.default)(API_ROUTE_PATH);
	server.route(routes);

	next();
}

register.attributes = {
	name: API_PROXY_PLUGIN_NAME,
	version: '1.0.0',
	dependencies: ['requestAuth', // provides request.plugins.requestAuth.oauth_token
	'tracking', // provides request.trackActivity()
	'electrode-csrf-jwt']
};
//# sourceMappingURL=index.js.map