'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/zip');

var _ = require('./');

var _duotone = require('./util/duotone');

var _send = require('./util/send');

var _receive = require('./util/receive');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
// Implicit dependency: tracking plugin providing request.trackActivity method


/*
 * This function transforms a single request to the application server into a
 * parallel array of requests to the API server, and then re-assembles the
 * API responses into an array of 'query responses' - i.e. API responses that
 * are formatted with properties from their corresponding query (ref, type).
 *
 * The logic for sending the requests is in './util/send' and the logic for
 * receiving the responses is in './util/receive'
 */
exports.default = function (request) {
	var setApiResponseDuotones = (0, _duotone.apiResponseDuotoneSetter)(request.server.plugins[_.API_PROXY_PLUGIN_NAME].duotoneUrls);
	return function (queries) {
		// send$ and receive must be assigned here rather than when the `request`
		// is first passed in because the `request.state` isn't guaranteed to be
		// available until after the `queries` have been parsed
		var send$ = (0, _send.makeSend$)(request);
		var receive = (0, _receive.makeReceive)(request);

		// create an array of in-flight API request Observables
		var apiRequests$ = queries.map(function (query) {
			return send$(query).map(receive(query)).map(setApiResponseDuotones);
		});

		// zip them together to make requests in parallel and return responses in order
		// $FlowFixMe - .zip is not currently defined in Observable static properties
		return _Observable.Observable.zip.apply(_Observable.Observable, _toConsumableArray(apiRequests$)).do(request.trackActivity);
	};
};
//# sourceMappingURL=proxy.js.map