'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _handler = require('./handler');

var _handler2 = _interopRequireDefault(_handler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import allows easier mocking in integration tests
var validApiPayloadSchema = _joi2.default.object({
	queries: _joi2.default.string().required(), // should be rison.encode_array-encoded
	metadata: _joi2.default.string(),
	logout: _joi2.default.any()
});

var getApiProxyRoutes = function getApiProxyRoutes(path) {
	/**
  * This handler converts the application-supplied queries into external API
  * calls, and converts the API call responses into a standard format that
  * the application expects.
  *
  * @returns Array query responses, which are in the format defined
  *   by `apiAdapter.apiResponseToQueryResponse`
  */
	var routeBase = {
		path: path,
		handler: _handler2.default,
		config: {
			plugins: {
				'electrode-csrf-jwt': {
					enabled: true
				}
			},
			state: {
				failAction: 'ignore' // ignore cookie validation, just accept
			}
		}
	};
	var apiGetRoute = _extends({}, routeBase, {
		method: ['GET', 'DELETE'],
		config: _extends({}, routeBase.config, {
			validate: {
				query: validApiPayloadSchema
			}
		})
	});
	var apiPostRoute = _extends({}, routeBase, {
		method: ['POST', 'PATCH'],
		config: _extends({}, routeBase.config, {
			payload: {
				allow: ['application/x-www-form-urlencoded', 'multipart/form-data'],
				multipart: {
					output: 'file' // parse file uploads into streams
				},
				maxBytes: 1024 * 1024 * 10 // 10 MB max upload
			}
		})
	});

	return [apiGetRoute, apiPostRoute];
};

exports.default = getApiProxyRoutes;
//# sourceMappingURL=routes.js.map