'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.makeSend$ = exports.makeExternalApiRequest = exports.makeMockRequest = exports.buildRequestArgs = exports.createCookieJar = exports.API_META_HEADER = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.getAuthHeaders = getAuthHeaders;
exports.getLanguageHeader = getLanguageHeader;
exports.getClientIpHeader = getClientIpHeader;
exports.parseRequestHeaders = parseRequestHeaders;
exports.parseMultipart = parseMultipart;
exports.getExternalRequestOpts = getExternalRequestOpts;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/of');

require('rxjs/add/observable/bindNodeCallback');

require('rxjs/add/observable/defer');

require('rxjs/add/operator/catch');

require('rxjs/add/operator/do');

require('rxjs/add/operator/map');

require('rxjs/add/operator/timeout');

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var API_META_HEADER = exports.API_META_HEADER = 'X-Meta-Request-Headers';

var MOCK_RESPONSE_OK = {
	// minimal representation of http.IncomingMessage
	statusCode: 200,
	statusMessage: 'OK',
	headers: {
		'x-meetup-request-id': 'mock request'
	}
};

/*
 * When an API response is being mocked (dev only), this response will be used
 */
function makeMockResponseOk(requestOpts) {
	return _extends({}, MOCK_RESPONSE_OK, {
		request: {
			uri: _url2.default.parse(requestOpts.url),
			method: requestOpts.method || 'get'
		}
	});
}

/**
 * In order to receive cookies from `externalRequest` requests, this function
 * provides a cookie jar that is specific to the request.
 *
 * The `requestUrl` is used to determine whether a cookie jar is needed
 *
 * https://github.com/request/request#examples
 *
 * @param {String} requestUrl the URL that will be used in the external request
 * @return {Object} a cookie jar compatible with the npm request `jar` API
 */
var createCookieJar = exports.createCookieJar = function createCookieJar(requestUrl) {
	var parsedUrl = _url2.default.parse(requestUrl);
	if (parsedUrl.pathname === '/sessions') {
		return _request2.default.jar(); // create request/url-specific cookie jar
	}
	return null;
};

/**
 * Transform each query into the arguments needed for a `request` call.
 *
 * Some request options are constant for all queries, and these are curried into
 * a function that can be called with a single query as part of the request
 * stream
 *
 * @see {@link https://www.npmjs.com/package/request}
 *
 * @param {Object} externalRequestOpts request options that will be applied to
 *   every query request
 * @param {Object} query { endpoint, params, flags }
 *   call)
 * @return {Object} externalRequestOptsQuery argument for the call to
 *   `externalRequest` for the query
 */
var buildRequestArgs = exports.buildRequestArgs = function buildRequestArgs(externalRequestOpts) {
	return function (_ref) {
		var endpoint = _ref.endpoint,
		    params = _ref.params,
		    flags = _ref.flags,
		    _ref$meta = _ref.meta,
		    meta = _ref$meta === undefined ? {} : _ref$meta;

		var dataParams = _qs2.default.stringify(params);
		var headers = _extends({}, externalRequestOpts.headers);
		var url = encodeURI('/' + endpoint);
		var body = void 0;
		var jar = createCookieJar(url);

		if (flags || meta.flags) {
			headers['X-Meetup-Request-Flags'] = (flags || meta.flags).join(',');
		}

		if (meta.metaRequestHeaders) {
			headers[API_META_HEADER] = meta.metaRequestHeaders.join(',');
		}

		if (meta.variants) {
			headers['X-Meetup-Variants'] = Object.keys(meta.variants).reduce(function (header, experiment) {
				var context = meta.variants[experiment];
				var contexts = context instanceof Array ? context : [context];
				header += contexts.map(function (c) {
					return experiment + '=' + c;
				}).join(' ');
				return header;
			}, '');
		}

		switch (externalRequestOpts.method) {
			case 'patch':
			case 'post':
				if (externalRequestOpts.formData) {
					break;
				}
				body = dataParams;
				headers['content-type'] = 'application/x-www-form-urlencoded';
				break;
			case 'delete':
			case 'get':
			default:
				url += '?' + dataParams;
				headers['content-type'] = 'application/json';
				headers['X-Meta-Photo-Host'] = 'secure';
		}

		var externalRequestOptsQuery = _extends({}, externalRequestOpts, {
			headers: headers,
			jar: jar,
			url: url
		});

		// only add body if defined
		if (body) {
			externalRequestOptsQuery.body = body;
		}

		return externalRequestOptsQuery;
	};
};

function getAuthHeaders(request) {
	var oauth_token = request.state.oauth_token || request.plugins.requestAuth.oauth_token; // browser-based requests // internal server requests
	if (!request.state.MEETUP_MEMBER && oauth_token) {
		return {
			authorization: 'Bearer ' + oauth_token
		};
	}
	var cookies = _extends({}, request.state);
	var csrf = _uuid2.default.v4();
	cookies.MEETUP_CSRF = csrf;
	cookies.MEETUP_CSRF_DEV = csrf;
	var cookie = Object.keys(cookies).map(function (name) {
		return name + '=' + cookies[name];
	}).join('; ');

	return {
		cookie: cookie,
		'csrf-token': csrf
	};
}

function getLanguageHeader(request) {
	var requestLang = request.getLanguage();
	var headerLang = request.headers['accept-language'];
	var acceptLang = requestLang && headerLang ? requestLang + ',' + headerLang : requestLang || headerLang;
	return acceptLang;
}

function getClientIpHeader(request) {
	var clientIP = request.headers['fastly-client-ip'];
	if (clientIP) {
		return { 'X-Meetup-Client-Ip': clientIP };
	}
}

function parseRequestHeaders(request) {
	var externalRequestHeaders = _extends({}, request.headers, getAuthHeaders(request), {
		'accept-language': getLanguageHeader(request)
	}, getClientIpHeader(request));

	delete externalRequestHeaders['host']; // let app server set 'host'
	delete externalRequestHeaders['accept-encoding']; // let app server set 'accept'
	delete externalRequestHeaders['content-length']; // original request content-length is irrelevant
	delete externalRequestHeaders['content-type']; // the content type will be set in buildRequestArgs

	return externalRequestHeaders;
}

/*
 * In multipart form requests, the parsed payload includes string key-value
 * pairs for regular inputs, and file descriptors for file upload inputs
 * ({ filename, path, headers }). This function passes through regular input
 * values unchanged, but converts the file descriptors to readable file streams
 * that will be proxied to the REST API.
 * 
 * References to the uploaded files are stored here for cleanup later on.
 */
function parseMultipart(payload, uploadsList) {
	return Object.keys(payload).reduce(function (formData, key) {
		var value = payload[key];
		if (value.filename) {
			formData[key] = {
				value: _fs2.default.createReadStream(value.path),
				options: {
					filename: value.filename,
					contentType: value.headers['content-type']
				}
			};
			uploadsList.push(value.path);
		} else {
			formData[key] = value;
		}
		return formData;
	}, {});
}

/*
 * Translate the incoming Hapi request into an 'opts' object that can be used
 * to call `request` for the REST API. This function essentially translates the
 * app server request properties (headers, payload) into corresponding REST API
 * request properties. The resulting value will be used as the _base_ set of
 * options for _every_ parallel REST API request made by the platform
 * corresponding to single incoming request.
 * 
 * @return {Object} externalRequestOpts
 */
function getExternalRequestOpts(request) {
	var baseUrl = request.server.settings.app.api.root_url;
	var externalRequestOpts = {
		baseUrl: baseUrl,
		method: request.method,
		headers: parseRequestHeaders(request), // make a copy to be immutable
		mode: 'no-cors',
		time: true, // time the request for logging
		agentOptions: {
			rejectUnauthorized: baseUrl.indexOf('.dev') === -1
		}
	};
	if (request.mime === 'multipart/form-data') {
		// multipart form data needs special treatment
		externalRequestOpts.formData = parseMultipart(request.payload, request.plugins[_.API_PROXY_PLUGIN_NAME].uploads);
	}
	return externalRequestOpts;
}

/**
 * Fake an API request and directly return the stringified mockResponse
 */
var makeMockRequest = exports.makeMockRequest = function makeMockRequest(mockResponse) {
	return function (requestOpts) {
		return _Observable.Observable.of([makeMockResponseOk(requestOpts), JSON.stringify(mockResponse)]).do(function () {
			return console.log('MOCKING response to ' + requestOpts.url);
		});
	};
};

var externalRequest$ = _Observable.Observable.bindNodeCallback(_request2.default);
/**
 * Make a real external API request, return response body string
 */
var makeExternalApiRequest = exports.makeExternalApiRequest = function makeExternalApiRequest(request) {
	return function (requestOpts) {
		return externalRequest$(requestOpts).do(
		// log errors
		null, function (err) {
			console.error(JSON.stringify({
				err: err.stack,
				message: 'REST API request error',
				request: {
					id: request.id
				},
				context: requestOpts
			}));
		}).timeout(request.server.settings.app.api.timeout).map(function (_ref2) {
			var _ref3 = _slicedToArray(_ref2, 2),
			    response = _ref3[0],
			    body = _ref3[1];

			return [response, body, requestOpts.jar];
		});
	};
};

var logOutgoingRequest = function logOutgoingRequest(request, requestOpts) {
	var method = requestOpts.method,
	    headers = requestOpts.headers;


	var parsedUrl = _url2.default.parse(requestOpts.url);
	request.server.app.logger.info({
		type: 'request',
		direction: 'out',
		info: {
			headers: headers,
			url: parsedUrl,
			method: method
		}
	}, 'Outgoing request ' + requestOpts.method.toUpperCase() + ' ' + parsedUrl.pathname);
};

/*
 * Make an API request and parse the response into the expected `response`
 * object shape
 */
var makeSend$ = exports.makeSend$ = function makeSend$(request) {
	// 1. get the queries and the shared `externalRequestOpts` from the request
	//    that will be applied to all queries
	var externalRequestOpts = getExternalRequestOpts(request);

	// 2. create a function that uses `externalRequestOpts` as a base from which
	//    to build query-specific API request options objects
	var queryToRequestOpts = buildRequestArgs(externalRequestOpts);

	return function (query) {
		var requestOpts = queryToRequestOpts(query);
		var request$ = query.mockResponse ? makeMockRequest(query.mockResponse) : makeExternalApiRequest(request);

		return _Observable.Observable.defer(function () {
			logOutgoingRequest(request, requestOpts);
			return request$(requestOpts);
		});
	};
};
//# sourceMappingURL=send.js.map