'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * This is the Hapi route handler that will be applied to the React application
 * route (usually a wildcard handling all paths).
 *
 * It is primarily responsible for two things:
 * 1. Ensure that the current URL pathname matches the language specified by the
 *    request - redirect if not
 * 2. Render the application in the requested language
 *
 * It also calls the tracking plugin in order to handle session tracking.
 */
exports.default = function (languageRenderers) {
	return function (request, reply) {
		var pathname = request.getLangPrefixPath();
		if (pathname !== request.url.pathname) {
			return reply.redirect(_url2.default.format(_extends({}, request.url, { pathname: pathname })));
		}
		var requestLanguage = request.getLanguage();
		var renderRequest = languageRenderers[requestLanguage];

		renderRequest(request).subscribe(function (renderResult) {
			if (renderResult.redirect) {
				return reply.redirect(renderResult.redirect.url).permanent(Boolean(renderResult.redirect.permanent));
			}
			return reply(renderResult.result).code(renderResult.statusCode);
		}, function (err) {
			return reply(err);
		} // 500 error - will only be thrown on bad implementation
		);
	};
};
//# sourceMappingURL=handler.js.map