'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.onPreResponse = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

var _handler = require('./handler');

var _handler2 = _interopRequireDefault(_handler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var onPreResponse = exports.onPreResponse = {
	/*
  * This function processes the route response before it is sent to the client.
  *
  * - In dev, it transforms the generic 500 error response JSON into a full dev-
  *   friendly rendering of the stack trace.
  */
	method: function method(request, reply) {
		var response = request.response;
		if (!response.isBoom || process.env.NODE_ENV === 'production') {
			return reply.continue();
		}
		var error = response;

		var _require = require('redbox-react'),
		    RedBoxError = _require.RedBoxError;

		var errorMarkup = _server2.default.renderToString(_react2.default.createElement(RedBoxError, { error: error }));
		var errorResponse = reply('<!DOCTYPE html><html><body>' + errorMarkup + '</body></html>');
		errorResponse.code(error.output.statusCode);
		return errorResponse;
	}
};

/*
 * Wildcard route for all application GET requests - see the 'handler' module
 * for rendering details
 *
 * Route config:
 *
 * - `onPreResponse`: process the response before it is sent to client
 * - `state`: skip cookie validation because cookies can be set by other services
 *   on the same domain and may not conform to the validation rules that are
 *   used by Hapi
 */

exports.default = function (languageRenderers) {
	return {
		method: 'GET',
		path: '/{wild*}',
		config: {
			ext: {
				onPreResponse: onPreResponse
			},
			plugins: {
				'electrode-csrf-jwt': {
					enabled: true // need to generate tokens on page request
				}
			},
			state: {
				failAction: 'ignore' // ignore cookie validation, just accept
			}
		},
		handler: (0, _handler2.default)(languageRenderers)
	};
};
//# sourceMappingURL=route.js.map