'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

/*
 * create a function that, when called, will determine the desired
 * language-prefixed path for the request. This can be used to redirect the
 * request to the 'correct' URL.
 * 
 * If the current request.url.pathname is correct, it will be returned
 * unmodified
 */
exports.default = function (request) {
	return function () {
		var supportedLangs = request.server.settings.app.supportedLangs;

		var requestLanguage = request.getLanguage();
		var originalPath = request.url.pathname;
		var firstPathComponent = originalPath.split('/')[1];
		// first listed locale is default
		if (requestLanguage === supportedLangs[0]) {
			// ensure that we are serving from un-prefixed URL
			if (supportedLangs.includes(firstPathComponent)) {
				var prefixedPath = originalPath.replace('/' + firstPathComponent, '');
				request.log(['info'], 'Unncessary lang path prefix (' + firstPathComponent + ')');
				return prefixedPath;
			}
		} else if (requestLanguage !== firstPathComponent) {
			// must correct/insert the correct lang prefix
			var cleanOriginal = originalPath.replace(new RegExp('^/(' + supportedLangs.join('|') + ')/'), '/');
			return '/' + requestLanguage + cleanOriginal;
		}
		return request.url.pathname;
	};
};
//# sourceMappingURL=getLangPrefixPath.js.map