'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getBrowserLang = exports.getUrlLang = exports.getCookieLang = undefined;

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _accepts = require('accepts');

var _accepts2 = _interopRequireDefault(_accepts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getServerSettings = function getServerSettings(request) {
	return request.server.settings.app;
};

/*
 * Get cookie-specified language using MEETUP_LANGUAGE cookie value
 */
var getCookieLang = exports.getCookieLang = function getCookieLang(request) {
	var _getServerSettings = getServerSettings(request),
	    isProd = _getServerSettings.isProd,
	    supportedLangs = _getServerSettings.supportedLangs;

	var LANGUAGE_COOKIE = isProd ? 'MEETUP_LANGUAGE' : 'MEETUP_LANGUAGE_DEV';

	var cookie = request.state[LANGUAGE_COOKIE];
	if (!cookie) {
		return;
	}

	var _querystring$parse = _qs2.default.parse(cookie),
	    language = _querystring$parse.language,
	    country = _querystring$parse.country;

	var cookieLang = country ? language + '-' + country : language;
	return supportedLangs.includes(cookieLang) && cookieLang;
};

/*
 * get the URL-specified language - check for existing language path prefixes
 */
var getUrlLang = exports.getUrlLang = function getUrlLang(request) {
	var _getServerSettings2 = getServerSettings(request),
	    supportedLangs = _getServerSettings2.supportedLangs;

	var urlLang = request.url.path.split('/')[1];
	return supportedLangs.includes(urlLang) && urlLang;
};

/*
 * Get browser-request language using accept-language header
 */
var getBrowserLang = exports.getBrowserLang = function getBrowserLang(request) {
	var _getServerSettings3 = getServerSettings(request),
	    supportedLangs = _getServerSettings3.supportedLangs;

	return (0, _accepts2.default)(request).language(supportedLangs);
};

/*
 * Get the language code determined from properties of the request (cookie,
 * url, header)
 */

exports.default = function (request) {
	return function () {
		return (
			// return the first language hit in the order of preference
			getCookieLang(request) || getUrlLang(request) || getBrowserLang(request) || getServerSettings(request).supportedLangs[0]
		);
	};
};
//# sourceMappingURL=getLanguage.js.map