'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = register;
var buildPaths = require('mwp-cli/src/config').paths;
var path = require('path');

/*
 * The service worker plugin will register a root-level route for serving the
 * service worker script file.
 *
 * Disk location of the script file is determined by builder CLI config import.
 */
function register(server, options, next) {
	/*
  * Route for service worker script at top-level path. Depends on `Inert`
  * `path` must match client `serviceWorker.register` call - MWP provides this
  * in the `<ServiceWorker>` component
  */
	server.route({
		method: 'GET',
		path: '/asset-service-worker.{localeCode}.js',
		config: {
			auth: false
		},
		handler: function handler(request, reply) {
			var localeCode = request.params.localeCode;

			var swPath = path.resolve(buildPaths.output.browser, localeCode, 'asset-service-worker.js');
			reply.file(swPath).type('application/javascript');
		}
	});

	next();
}

register.attributes = {
	name: 'mwp-service-worker',
	version: '1.0.0',
	dependencies: 'inert' // decorates `reply.file`
};
//# sourceMappingURL=index.js.map