'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getTrackActivity = exports.getTrackApiResponses = undefined;

var _url2 = require('url');

var _url3 = _interopRequireDefault(_url2);

var _rison = require('rison');

var _rison2 = _interopRequireDefault(_rison);

var _idUtils = require('./util/idUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parseUrl = _url3.default.parse;

/*
 * This module exports specific tracking functions that consume the `request`
 * object and any additional arguments that are relevant to generating the
 * tracking record.
 */

var getTrackApiResponses = exports.getTrackApiResponses = function getTrackApiResponses(trackOpts) {
	return function (request) {
		return function (queryResponses, url, referrer) {
			var apiRequests = queryResponses.map(function (_ref) {
				var meta = _ref.meta;
				return {
					requestId: meta.requestId,
					endpoint: meta.endpoint
				};
			});
			return trackOpts.log(request, {
				description: 'nav',
				memberId: (0, _idUtils.parseIdCookie)(request.state[trackOpts.memberCookieName], true), // read memberId
				browserId: (0, _idUtils.updateId)(trackOpts.browserIdCookieName)(request), // read/add browserId
				trackId: (0, _idUtils.updateId)(trackOpts.trackIdCookieName)(request), // read/add trackId
				url: url || '',
				referer: referrer || '',
				apiRequests: apiRequests
			});
		};
	};
};

/*
 * This is the core tracking handler - called on every request that generates
 * REST API call(s)
 */
var getTrackActivity = exports.getTrackActivity = function getTrackActivity(trackOpts) {
	return function (request) {
		return function (queryResponses) {
			var method = request.method,
			    payload = request.payload,
			    query = request.query,
			    referrer = request.info.referrer;

			var requestReferrer = parseUrl(referrer).pathname || '';

			var reqData = method === 'post' ? payload : query;

			if (reqData.metadata) {
				// this is an API-proxy request, so the referrer needs to be read from
				// the reqData. The 'url' is the 'requestReferrer'
				var metadataRison = reqData.metadata || _rison2.default.encode_object({});

				var _rison$decode_object = _rison2.default.decode_object(metadataRison),
				    _referrer = _rison$decode_object.referrer;

				var _url = parseUrl(requestReferrer).pathname;
				return request.trackApiResponses(queryResponses, _url, _referrer);
			}

			return request.trackApiResponses(queryResponses, request.url.pathname, // requested url
			requestReferrer // referer
			);
		};
	};
};
//# sourceMappingURL=_activityTrackers.js.map