'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.clickToClickRecord = exports.clickCookieOptions = undefined;
exports.default = processClickTracking;

var _avro = require('./avro');

var _avro2 = _interopRequireDefault(_avro);

var _cookieUtils = require('../../../util/cookieUtils');

var _clickState = require('./clickState');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * This module provides utilities for reading click cookie data sent as part of
 * a request to the application - it should be set up as a server plugin
 */

var isProd = process.env.NODE_ENV === 'production'; // TODO: provide this info through new plugin
var clickCookieOptions = exports.clickCookieOptions = {
	isSecure: isProd,
	isHttpOnly: false,
	domain: (isProd ? '' : '.dev') + '.meetup.com'
};

var clickToClickRecord = exports.clickToClickRecord = function clickToClickRecord(request) {
	return function (click) {
		return {
			timestamp: click.timestamp || new Date().toISOString(),
			requestId: request.id,
			memberId: (0, _cookieUtils.parseMemberCookie)(request.state).id,
			lineage: click.lineage,
			linkText: click.linkText || '',
			coordX: click.coords[0],
			coordY: click.coords[1]
		};
	};
};

function processClickTracking(request, reply) {
	var rawCookieValue = (request.state || {})[_clickState.COOKIE_NAME];
	// It's possible that multiple cookies with the same value were sent, e.g.
	// one value for .dev.meetup.com and another for .meetup.com - parse only the first
	var cookieValue = rawCookieValue instanceof Array ? rawCookieValue[0] : rawCookieValue;
	if (!cookieValue || cookieValue === 'undefined') {
		return;
	}

	var cookieJSON = decodeURIComponent(cookieValue);

	var _JSON$parse = JSON.parse(cookieJSON),
	    history = _JSON$parse.history;

	history.map(clickToClickRecord(request)).forEach(_avro2.default.loggers.click);

	reply.unstate(_clickState.COOKIE_NAME, clickCookieOptions);
	return;
}
//# sourceMappingURL=clickReader.js.map