'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.DEFAULT_CLICK_TRACK = exports.actions = exports.CLICK_TRACK_CLEAR_ACTION = exports.CLICK_TRACK_ACTION = exports.setClickCookie = exports.COOKIE_NAME = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.reducer = reducer;

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/*
 * This module provides utilities for managing click tracking data in Redux
 * state an saving that data to a browser cookie
 */

var COOKIE_NAME = exports.COOKIE_NAME = 'click-track'; // must remain in sync with Meetup Classic implementation

var BrowserCookies = _jsCookie2.default.withConverter({
	read: function read(value) {
		return value;
	},
	write: function write(value) {
		return encodeURIComponent(value).replace(/[!'()*]/g, function (c) {
			return '%' + c.charCodeAt(0).toString(16);
		});
	}
});

var setClickCookie = exports.setClickCookie = function setClickCookie(clickTracking) {
	var domain = window.location.host.replace(/[^.]+/, ''); // strip leading subdomain, e.g. www or beta2
	var cookieVal = JSON.stringify(clickTracking);
	BrowserCookies.set(COOKIE_NAME, cookieVal, { domain: domain });
};

var CLICK_TRACK_ACTION = exports.CLICK_TRACK_ACTION = 'CLICK_TRACK';
var CLICK_TRACK_CLEAR_ACTION = exports.CLICK_TRACK_CLEAR_ACTION = 'CLICK_TRACK_CLEAR';

var actions = exports.actions = {
	click: function click(clickData) {
		return {
			type: CLICK_TRACK_ACTION,
			payload: clickData
		};
	},
	clear: function clear() {
		return { type: CLICK_TRACK_CLEAR_ACTION };
	}
};

var DEFAULT_CLICK_TRACK = exports.DEFAULT_CLICK_TRACK = { history: [] };
/**
 * @param {Object} data extensible object to store click data {
 *   history: array
 * }
 * @param {Object} action the dispatched action
 * @return {Object} new state
 */
function reducer() {
	var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_CLICK_TRACK;
	var action = arguments[1];

	if (action.type === CLICK_TRACK_ACTION) {
		var history = [].concat(_toConsumableArray(state.history), [action.payload]);
		return _extends({}, state, {
			history: history
		});
	}
	if (action.type === CLICK_TRACK_CLEAR_ACTION) {
		return DEFAULT_CLICK_TRACK;
	}
	return state;
}
//# sourceMappingURL=clickState.js.map