'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.parseIdCookie = exports.makeIdCookie = exports.newId = exports.updateId = undefined;

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _activity = require('../activity');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * This is a 'get or set' function for the `cookieName` passed in.
 */
var updateId = exports.updateId = function updateId(cookieName) {
	return function (request, doRefresh) {
		var cookieVal = request.state[cookieName] || // cookie in original request
		request.plugins[_activity.ACTIVITY_PLUGIN_NAME][cookieName]; // cookie added to outgoing response

		if (!cookieVal || doRefresh) {
			// Generate a new id value and store in request. Cookie will be
			// set in the plugin's onResponse handler
			return newId(cookieName)(request);
		}
		return parseIdCookie(cookieVal).toString(); // toString used to satisfy Flow
	};
};

/*
 * This function creates a new uuid and stores it in the request using a
 * `cookieName` key. The actual cookie will be set in the plugin's `onResponse`
 * handler in order to share the cookie across browser tabs.
 */

var newId = exports.newId = function newId(cookieName) {
	return function (request) {
		var id = _uuid2.default.v4();
		request.plugins[_activity.ACTIVITY_PLUGIN_NAME][cookieName] = makeIdCookie(id);
		return id;
	};
};

var makeIdCookie = exports.makeIdCookie = function makeIdCookie(id) {
	return 'id=' + id;
};
var parseIdCookie = exports.parseIdCookie = function parseIdCookie(cookieVal, doParseInt) {
	var parsed = _qs2.default.parse(cookieVal) || { id: '' };
	parsed.id = parsed.id || '';
	if (doParseInt) {
		return parseInt(parsed.id, 10) || 0;
	}
	return parsed.id;
};
//# sourceMappingURL=idUtils.js.map