'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.resolveAppProps = resolveAppProps;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _browser = require('../store/browser');

var _util = require('../router/util');

var _BrowserApp = require('../render/components/BrowserApp');

var _BrowserApp2 = _interopRequireDefault(_BrowserApp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Async resolver of the props needed for `BrowserApp`:
 * { routes, store, basename }
 */


/**
 * @module browser-render
 */

function resolveAppProps(routes, reducer) {
	var middleware = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

	var basename = window.APP_RUNTIME.baseUrl || '';
	var createStore = (0, _browser.getBrowserCreateStore)(routes, middleware, basename);
	var store = createStore(reducer, (0, _browser.getInitialState)(window.APP_RUNTIME));
	return (0, _util.getRouteResolver)(routes, basename)(window.location).then(function () {
		return {
			routes: routes,
			store: store,
			basename: basename
		};
	});
}

/**
 * This function creates a 'renderer', which is just a function that, when
 * called, will call ReactDOM.render() to render the application
 *
 * The routes, reducer, and app-specific middleware are provided by the
 * application - everything else is general to the meetup web platform
 *
 * @deprecated see CHANGELOG v2.4
 */

function makeRenderer(routes, reducer) {
	var middleware = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

	return function () {
		var rootElId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'outlet';

		resolveAppProps(routes, reducer, middleware).then(function (props) {
			_reactDom2.default.render(_react2.default.createElement(_BrowserApp2.default, props), document.getElementById(rootElId));
			return props.store;
		});
	};
}

exports.default = makeRenderer;
//# sourceMappingURL=browser-render.js.map