'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.makeRenderer = exports.makeRenderer$ = undefined;

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/of');

require('rxjs/add/operator/first');

var _intl = require('intl');

var _intl2 = _interopRequireDefault(_intl);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

var _reactHelmet = require('react-helmet');

var _reactHelmet2 = _interopRequireDefault(_reactHelmet);

var _apiProxy = require('../plugins/api-proxy');

var _router = require('../router');

var _server3 = require('../store/server');

var _Dom = require('../render/components/Dom');

var _Dom2 = _interopRequireDefault(_Dom);

var _ServerApp = require('../render/components/ServerApp');

var _ServerApp2 = _interopRequireDefault(_ServerApp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// mwp-render/components/ServerApp

// Ensure global Intl for use with FormatJS
// mwp-store
// mwp-api-proxy
Intl.NumberFormat = _intl2.default.NumberFormat; // mwp-render/components/Dom
// mwp-router

Intl.DateTimeFormat = _intl2.default.DateTimeFormat;

var DOCTYPE = '<!DOCTYPE html>';

/**
 * An async module that renders the full app markup for a particular URL/location
 * using [ReactDOMServer]{@link https://facebook.github.io/react/docs/top-level-api.html#reactdomserver}
 *
 * @module ServerRender
 */

function getHtml(el) {
	var htmlMarkup = _server2.default.renderToString(el);
	return '' + DOCTYPE + htmlMarkup;
}

function getRedirect(context) {
	if (!context || !context.url) {
		return;
	}
	return {
		redirect: {
			url: context.url,
			permanent: context.permanent
		}
	};
}

/*
 * A helper to collect react-side-effect data from all components in the
 * application that utilize side effects
 *
 * Any component class that uses react-side-effect should be 'rewound' in this
 * function to prevent memory leaks and invalid state carrying over into other
 * requests
 */
var resolveSideEffects = function resolveSideEffects() {
	return {
		head: _reactHelmet2.default.rewind(),
		redirect: _router.Redirect.rewind(),
		forbidden: _router.Forbidden.rewind(),
		notFound: _router.NotFound.rewind()
	};
};

/**
 * Using the current route information and Redux store, render the app to an
 * HTML string and server response code.
 *
 * There are three parts to the render:
 *
 * 1. `appMarkup`, which corresponds to the markup that will be rendered
 * on the client by React. This string is built before the full markup because
 * it sets the data needed by other parts of the DOM, such as `<head>`.
 * 2. `htmlMarkup`, which wraps `appMarkup` with the remaining DOM markup.
 * 3. `doctype`, which is just the doctype element that is a sibling of `<html>`
 *
 * @param {Object} renderProps
 * @param {ReduxStore} store the store containing the initial state of the app
 * @return {Object} the statusCode and result used by Hapi's `reply` API
 *   {@link http://hapijs.com/api#replyerr-result}
 */


var getRouterRenderer = function getRouterRenderer(_ref) {
	var routes = _ref.routes,
	    store = _ref.store,
	    location = _ref.location,
	    baseUrl = _ref.baseUrl,
	    assetPublicPath = _ref.assetPublicPath,
	    scripts = _ref.scripts,
	    cssLinks = _ref.cssLinks;

	// pre-render the app-specific markup, this is the string of markup that will
	// be managed by React on the client.
	//
	// **IMPORTANT**: this string is built separately from `<Dom />` because it
	// initializes page-specific state that `<Dom />` needs to render, e.g.
	// `<head>` contents
	var initialState = store.getState();
	var appMarkup = void 0;
	var staticContext = {};

	try {
		appMarkup = _server2.default.renderToString(_react2.default.createElement(_ServerApp2.default, {
			basename: baseUrl,
			location: location,
			context: staticContext,
			store: store,
			routes: routes
		}));
	} catch (err) {
		// cleanup all react-side-effect components to prevent error/memory leaks
		resolveSideEffects();
		// now we can re-throw and let the caller handle the error
		throw err;
	}

	var sideEffects = resolveSideEffects();

	var externalRedirect = getRedirect(sideEffects.redirect);
	var internalRedirect = getRedirect(staticContext);
	var redirect = internalRedirect || externalRedirect;
	if (redirect) {
		return redirect;
	}

	// all the data for the full `<html>` element has been initialized by the app
	// so go ahead and assemble the full response body
	var result = getHtml(_react2.default.createElement(_Dom2.default, {
		baseUrl: baseUrl,
		assetPublicPath: assetPublicPath,
		head: sideEffects.head,
		initialState: initialState,
		appMarkup: appMarkup,
		scripts: scripts,
		cssLinks: cssLinks
	}));

	// prioritized status code fallbacks
	var statusCode = sideEffects.forbidden || sideEffects.notFound || 200;

	return {
		statusCode: statusCode,
		result: result
	};
};

var makeRenderer$ = function makeRenderer$(renderConfig) {
	return makeRenderer(renderConfig.routes, renderConfig.reducer, null, renderConfig.assetPublicPath, renderConfig.middleware, renderConfig.baseUrl, renderConfig.scripts, renderConfig.enableServiceWorker, renderConfig.cssLinks);
};

/**
 * Curry a function that takes a Hapi request and returns an observable
 * that will emit the rendered HTML
 *
 * The outer function takes app-specific information about the routes,
 * reducer, and optional additional middleware
 *
 * @param {Object} routes the React Router routes object
 * @param {Function} reducer the root Redux reducer for the app
 * @param {Function} middleware (optional) any app-specific middleware that
 *   should be applied to the store
 *
 * @return {Function}
 *
 * -- Returned Fn --
 * @param {Request} request The request to render - must already have an
 * `oauth_token` in `state`
 * @return {Observable}
 */
var makeRenderer = function makeRenderer(routes, reducer, clientFilename, assetPublicPath) {
	var middleware = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
	var baseUrl = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : '';
	var scripts = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : [];
	var enableServiceWorker = arguments[7];
	var cssLinks = arguments[8];
	return function (request) {
		middleware = middleware || [];

		if (clientFilename) {
			console.warn('`clientFilename` deprecated in favor of `scripts` array in makeRenderer');
			scripts.push('' + assetPublicPath + clientFilename);
		}

		if (!scripts.length) {
			throw new Error('No client script assets specified');
		}

		var connection = request.connection,
		    headers = request.headers,
		    info = request.info,
		    url = request.url,
		    _request$server = request.server,
		    logger = _request$server.app.logger,
		    supportedLangs = _request$server.settings.app.supportedLangs,
		    req = request.raw.req;

		var requestLanguage = request.getLanguage();

		// request protocol and host might be different from original request that hit proxy
		// we want to use the proxy's protocol and host
		var requestProtocol = headers['x-forwarded-proto'] || connection.info.protocol;
		var domain = headers['x-forwarded-host'] || headers['x-meetup-host'] || info.host;
		var host = requestProtocol + '://' + domain;

		// create the store with populated `config`
		var initialState = {
			config: {
				apiUrl: _apiProxy.API_ROUTE_PATH,
				baseUrl: host,
				enableServiceWorker: enableServiceWorker,
				requestLanguage: requestLanguage,
				supportedLangs: supportedLangs,
				initialNow: new Date().getTime()
			}
		};

		var createStore = (0, _server3.getServerCreateStore)(routes, middleware, request, baseUrl);
		var store = createStore(reducer, initialState);

		// render skeleton if requested - the store is ready
		if ('skeleton' in request.query) {
			return _Observable.Observable.of({
				result: getHtml(_react2.default.createElement(_Dom2.default, {
					baseUrl: baseUrl,
					assetPublicPath: assetPublicPath,
					head: _reactHelmet2.default.rewind(),
					initialState: store.getState(),
					scripts: scripts,
					cssLinks: cssLinks
				})),
				statusCode: 200
			});
		}

		// otherwise render using the API and React router
		var storeIsReady$ = _Observable.Observable.create(function (obs) {
			obs.next(store.getState());
			return store.subscribe(function () {
				return obs.next(store.getState());
			});
		}).first(function (state) {
			return state.preRenderChecklist.every(function (isReady) {
				return isReady;
			});
		}); // take the first ready state

		var action = {
			type: _router.SERVER_RENDER,
			payload: url
		};
		logger.debug({ type: 'dispatch', action: action, req: req }, 'Dispatching RENDER for ' + request.url.href);
		store.dispatch(action);
		return storeIsReady$.map(function () {
			return getRouterRenderer({
				routes: routes,
				store: store,
				location: url,
				baseUrl: baseUrl,
				assetPublicPath: assetPublicPath,
				scripts: scripts,
				cssLinks: cssLinks
			});
		});
	};
};

exports.makeRenderer$ = makeRenderer$;
exports.makeRenderer = makeRenderer;
exports.default = makeRenderer;
//# sourceMappingURL=server-render.js.map