'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _util = require('./util');

var _RouteLayout = require('./RouteLayout');

var _RouteLayout2 = _interopRequireDefault(_RouteLayout);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Route rendering component that will render nested routes asynchronously
 * The nested routes are cached so that the async data is not re-requested
 * each time a route is re-rendered.
 */
var AsyncRoute = function (_React$Component) {
	_inherits(AsyncRoute, _React$Component);

	function AsyncRoute(props) {
		_classCallCheck(this, AsyncRoute);

		var _this = _possibleConstructorReturn(this, (AsyncRoute.__proto__ || Object.getPrototypeOf(AsyncRoute)).call(this, props));

		var match = props.match,
		    route = props.route;

		var childRoutes = (0, _util.getChildRoutes)({ match: match, route: route });
		_this.state = {
			childRoutes: childRoutes,
			_routesCache: {}
		};
		return _this;
	}

	_createClass(AsyncRoute, [{
		key: 'updateChildRoutes',
		value: function updateChildRoutes(childRoutes, key) {
			this.setState(function (state) {
				return {
					childRoutes: childRoutes,
					_routesCache: _extends({}, state._routesCache, _defineProperty({}, key, childRoutes))
				};
			});
		}
	}, {
		key: 'resolveAsyncChildRoutes',
		value: function resolveAsyncChildRoutes(resolver) {
			var _this2 = this;

			var key = resolver.toString();
			// async route - check for cache keyed by stringified load function
			var cached = this.state._routesCache[key];
			if (cached) {
				this.updateChildRoutes(cached, key);
				return;
			}
			resolver().then(function (routes) {
				routes = routes instanceof Array ? routes : [routes];
				_this2.updateChildRoutes(routes, key);
			});
			return;
		}
		/*
   * New props may correspond to a route change. If so, this function populates
   * this.state.childRoutes to correspond to the indexRoute or nested routes
   * defined by the new route
   */

	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			var match = nextProps.match,
			    route = nextProps.route;

			if (route === this.props.route && match === this.props.match) {
				return;
			}
			var childRoutes = (0, _util.getChildRoutes)({ match: match, route: route });
			this.setState(function (state) {
				return { childRoutes: childRoutes };
			});
			if (childRoutes.length) {
				return;
			}
			if (match.isExact && route.getIndexRoute) {
				this.resolveAsyncChildRoutes(route.getIndexRoute);
				return;
			}

			if (route.getNestedRoutes) {
				this.resolveAsyncChildRoutes(route.getNestedRoutes);
			}
			return;
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    route = _props.route,
			    match = _props.match,
			    props = _objectWithoutProperties(_props, ['route', 'match']);

			var childRoutes = this.state.childRoutes;
			// React Router encodes the URL params - send decoded values to component

			var params = (0, _util.decodeParams)(match.params);
			var Component = route.component;

			return _react2.default.createElement(
				Component,
				_extends({}, props, { route: route, match: _extends({}, match, { params: params }) }),
				childRoutes.length > 0 && _react2.default.createElement(_RouteLayout2.default, { routes: childRoutes, matchedPath: match.path })
			);
		}
	}]);

	return AsyncRoute;
}(_react2.default.Component);

exports.default = AsyncRoute;
//# sourceMappingURL=AsyncRoute.js.map