'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.activeRouteQueries = exports.getMatchedQueries = exports.decodeParams = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resolve = require('./resolve');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var decodeParams = exports.decodeParams = function decodeParams(params) {
	return Object.keys(params).reduce(function (decodedParams, key) {
		if (typeof params[key] !== 'undefined') {
			// skip 'undefined' values that cannot be encoded (null is okay)
			decodedParams[key] = params[key] && decodeURI(params[key]);
		}
		return decodedParams;
	}, {});
};

/*
 * find all routes from a given array of route config objects that match the
 * supplied `location`
 *
 * this function matches the signature of `react-router-config`'s `_resolveRouteMatches`
 * function, but interprets all `route.path` settings as nested
 *
 * @see {@link https://github.com/ReactTraining/react-router/tree/master/packages/react-router-config#matchroutesroutes-pathname}
 */
/*
 * Collect the return values of each query functions associated with the matched
 * routes, called with the provided location URL
 *
 * The function returned from calling `_matchedRouteQueriesReducer` with a
 * `location` should be used as the callback to an `array.reduce` call.
 */
var _matchedRouteQueriesReducer = function _matchedRouteQueriesReducer(location) {
	return function (queries, _ref) {
		var route = _ref.route,
		    match = _ref.match;

		if (!route.query) {
			return queries;
		}
		var routeQueryFns = route.query instanceof Array ? route.query : [route.query];

		// call the query functions with non-url-encoded params
		var params = decodeParams(match.params);
		var routeQueries = routeQueryFns.map(function (queryFn) {
			return queryFn(_extends({}, match, { location: location, params: params }));
		}).filter(function (query) {
			return query;
		});

		return [].concat(_toConsumableArray(queries), _toConsumableArray(routeQueries));
	};
};

/*
 * A synchronous, curried interface to derive the query values returned by the
 * query functions of a provided set of routes given a particular location
 */
var getMatchedQueries = exports.getMatchedQueries = function getMatchedQueries(location) {
	return function (matchedRoutes) {
		return matchedRoutes.reduce(_matchedRouteQueriesReducer(location), []);
	};
};

/*
 * A curried interface into `_resolveRouteMatches` + `getMatchedQueries`
 */
var activeRouteQueries = exports.activeRouteQueries = function activeRouteQueries(routes, baseUrl) {
	var resolveRoutes = (0, _resolve.getRouteResolver)(routes, baseUrl);
	return function (location) {
		return resolveRoutes(location).then(getMatchedQueries(location));
	};
};
//# sourceMappingURL=query.js.map