'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getRouteResolver = exports.resolveChildRoutes = exports.getChildRoutes = undefined;

var _matchPath = require('react-router-dom/matchPath');

var _matchPath2 = _interopRequireDefault(_matchPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
/**
 * Utilities for interacting with the Router and getting location data
 * @module routeUtils
 */


// munge a route's 'relative' `path` with the full matchedPath


// A type for functions that mutate an object asynchronously
var _routeMatchOptions = function _routeMatchOptions(route, matchedPath) {
	return {
		path: ('' + matchedPath + (route.path || '')).replace('//', '/'),
		strict: route.strict,
		exact: route.exact
	};
};

/*
 * Determine whether the indexRoute or nested route should be considered the
 * child route for a particular MatchedRoute
 */
var getChildRoutes = exports.getChildRoutes = function getChildRoutes(matchedRoute) {
	var route = matchedRoute.route,
	    match = matchedRoute.match;

	if (match.isExact) {
		return route.indexRoute ? [route.indexRoute] : [];
	}
	return route.routes || [];
};

/*
 * Given a matched route, return the expected child route(s). This function
 * encapsulates the logic for selecting between the `indexRoute` and the
 * child `routes` array
 *
 * This is essentially an async `getChildRoutes`
 */
var resolveChildRoutes = exports.resolveChildRoutes = function resolveChildRoutes(matchedRoute) {
	var match = matchedRoute.match;

	if (match.isExact) {
		return _resolveIndexRoute(matchedRoute).then(function (m) {
			return m.route.indexRoute ? [m.route.indexRoute] : [];
		});
	}
	return _resolveNestedRoutes(matchedRoute).then(function (m) {
		return m.route.routes || [];
	});
};

var _resolveNestedRoutes = function _resolveNestedRoutes(matchedRoute) {
	return matchedRoute.route.getNestedRoutes ? matchedRoute.route.getNestedRoutes().then(function (routes) {
		return matchedRoute.route.routes = routes;
	}).then(function () {
		return matchedRoute;
	}) : Promise.resolve(matchedRoute);
};

var _resolveIndexRoute = function _resolveIndexRoute(matchedRoute) {
	return matchedRoute.route.getIndexRoute ? matchedRoute.route.getIndexRoute().then(function (indexRoute) {
		return matchedRoute.route.indexRoute = indexRoute;
	}).then(function () {
		return matchedRoute;
	}) : Promise.resolve(matchedRoute);
};

/*
 * Find all routes in the routes array that match the provided URL, including
 * nested routes that may be async
 */
var _resolveRouteMatches = function _resolveRouteMatches() {
	var routes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
	var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
	var matchedRoutes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
	var matchedPath = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';

	var route = routes.find(function (r) {
		return (0, _matchPath2.default)(path, _routeMatchOptions(r, matchedPath));
	}); // take the first match
	if (!route) {
		return Promise.resolve(matchedRoutes);
	}

	// add the route and its `match` object to the array of matched routes
	var currentMatchOptions = _routeMatchOptions(route, matchedPath);
	var match = (0, _matchPath2.default)(path, currentMatchOptions);
	if (!match) {
		// we know that this won't ever run because we've established the match in
		// `.find`, but this check is for type safety
		return Promise.resolve(matchedRoutes);
	}
	var matchedRoute = { route: route, match: match };
	var currentMatchedRoutes = [].concat(_toConsumableArray(matchedRoutes), [matchedRoute]);

	// add any nested route matches
	return resolveChildRoutes(matchedRoute).then(function (childRoutes) {
		return childRoutes.length ? _resolveRouteMatches(childRoutes, path, currentMatchedRoutes, currentMatchOptions.path) : currentMatchedRoutes;
	});
};

/*
 * An curried interface into `_resolveRouteMatches`, using `baseUrl`
 * + `location` instead of `path`
 */
var getRouteResolver = exports.getRouteResolver = function getRouteResolver(routes, baseUrl) {
	return function (location) {
		var path = location.pathname.replace(baseUrl, '');
		return _resolveRouteMatches(routes, path);
	};
};
//# sourceMappingURL=resolve.js.map