'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getFetchArgs = exports.CSRF_HEADER_COOKIE = exports.CSRF_HEADER = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

var _rison = require('rison');

var _rison2 = _interopRequireDefault(_rison);

var _clickState = require('../../plugins/tracking/util/clickState');

var _fetchUtils = require('../util/fetchUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } // mwp-tracking-plugin

var CSRF_HEADER = exports.CSRF_HEADER = 'x-csrf-jwt';
var CSRF_HEADER_COOKIE = exports.CSRF_HEADER_COOKIE = 'x-csrf-jwt-header';

/**
 * that the request will have the required OAuth and CSRF credentials and constructs
 * the `fetch` call arguments based on the request method. It also records the
 * CSRF header value in a cookie for use as a CSRF header in future fetches.
 *
 * @param {String} apiUrl the general-purpose endpoint for API calls to the
 *   application server
 * @param {Array} queries the queries to send - must all use the same `method`
 * @param {Object} meta additional characteristics of the request, e.g.
 *   click tracking data
 * @return {Object} { url, config } arguments for a fetch call
 */
var getFetchArgs = function getFetchArgs(apiUrl, queries, meta) {
	var headers = {};
	var method = ((queries[0].meta || {}).method || 'GET'). // fallback to 'get'
	toUpperCase(); // must be upper case - requests can fail silently otherwise

	var hasBody = method === 'POST' || method === 'PATCH';
	var isFormData = queries[0].params instanceof FormData;
	var isDelete = method === 'DELETE';

	var searchParams = new URLSearchParams();
	searchParams.append('queries', _rison2.default.encode_array(queries));

	if (meta) {
		var clickTracking = meta.clickTracking,
		    onSuccess = meta.onSuccess,
		    onError = meta.onError,
		    metadata = _objectWithoutProperties(meta, ['clickTracking', 'onSuccess', 'onError']);

		if (clickTracking) {
			(0, _clickState.setClickCookie)(clickTracking);
		}

		// send other metadata in searchParams
		var encodedMetadata = metadata && _rison2.default.encode_object(metadata);
		if (encodedMetadata) {
			// send other metadata in searchParams
			searchParams.append('metadata', encodedMetadata);
		}
	}

	if (!isFormData) {
		// need to manually specify content-type for any non-multipart request
		headers['content-type'] = hasBody ? 'application/x-www-form-urlencoded' : 'application/json';
	}

	if (hasBody || isDelete) {
		headers[CSRF_HEADER] = _jsCookie2.default.get(CSRF_HEADER_COOKIE);
	}

	var config = {
		method: method,
		headers: headers,
		credentials: 'same-origin' // allow response to set-cookies
	};
	if (hasBody) {
		config.body = isFormData ? queries[0].params : searchParams.toString();
	}
	var useQueryString = isFormData || !hasBody;
	var url = useQueryString ? apiUrl + '?' + searchParams : apiUrl;
	return {
		url: url,
		config: config
	};
};

/**
 * Wrapper around `fetch` to send an array of queries to the server and organize
 * the responses.
*
 * **IMPORTANT**: This function should _only_ be called from the browser. The
 * server should never need to call itself over HTTP
 *
 * @param {String} apiUrl the general-purpose endpoint for API calls to the
 *   application server
 * @param {Array} queries the queries to send - must all use the same `method`
 * @param {Object} meta additional characteristics of the request, e.g.
 *   click tracking data
 * @return {Promise} resolves with a `{queries, responses}` object
 */
exports.getFetchArgs = getFetchArgs;
var fetchQueries = function fetchQueries(apiUrl) {
	return function (queries, meta) {
		if (typeof window === 'undefined' && typeof test === 'undefined' // not in browser // not in testing env (global set by Jest)
		) {
				throw new Error('fetchQueries was called on server - cannot continue');
			}

		var _getFetchArgs = getFetchArgs(apiUrl, queries, meta),
		    url = _getFetchArgs.url,
		    config = _getFetchArgs.config;

		return fetch(url, config).then(function (queryResponse) {
			return queryResponse.json();
		}).then(function (queryJSON) {
			return _extends({}, (0, _fetchUtils.parseQueryResponse)(queries)(queryJSON));
		}).catch(function (err) {
			console.error(JSON.stringify({
				err: err.stack,
				message: 'App server API fetch error',
				context: config
			}));
			throw err; // handle the error upstream
		});
	};
};

exports.default = fetchQueries;
//# sourceMappingURL=fetchQueries.js.map