'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getInitialState = exports.clickTrackEnhancer = undefined;
exports.getBrowserCreateStore = getBrowserCreateStore;

var _redux = require('redux');

var _clickWriter = require('../../plugins/tracking/util/clickWriter');

var _clickWriter2 = _interopRequireDefault(_clickWriter);

var _apiState = require('../../api-state');

var _catch = require('../middleware/catch');

var _catch2 = _interopRequireDefault(_catch);

var _injectPromise = require('../middleware/injectPromise');

var _injectPromise2 = _interopRequireDefault(_injectPromise);

var _fetchQueries = require('./fetchQueries');

var _fetchQueries2 = _interopRequireDefault(_fetchQueries);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } //  weak
// mwp-tracking/util/clickWriter


// ignore 'potentially null' document.body

var noopMiddleware = function noopMiddleware(store) {
	return function (next) {
		return function (action) {
			return next(action);
		};
	};
};

var clickTrackEnhancer = exports.clickTrackEnhancer = function clickTrackEnhancer(createStore) {
	return function (reducer, initialState, enhancer) {
		var store = createStore(reducer, initialState, enhancer);
		var clickTracker = (0, _clickWriter2.default)(store);
		document.body.addEventListener('click', clickTracker);
		document.body.addEventListener('change', clickTracker);

		return store;
	};
};

/**
 * the initial state is delivered in the HTML from the server as a plain object
 * containing the HTML-escaped JSON string in `window.INITIAL_STATE.escapedState`.
 * unescape the text using native `textarea.textContent` unescaping
 */
var getInitialState = exports.getInitialState = function getInitialState(APP_RUNTIME) {
	if (!APP_RUNTIME) {
		return;
	}
	var escape = document.createElement('textarea');
	escape.innerHTML = APP_RUNTIME.escapedState;
	var unescapedStateJSON = escape.textContent;
	return JSON.parse(unescapedStateJSON);
};

function getBrowserCreateStore(routes) {
	var middleware = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
	var baseUrl = arguments[2];

	var middlewareToApply = [(0, _catch2.default)(console.error), _injectPromise2.default, (0, _apiState.getApiMiddleware)(routes, _fetchQueries2.default, baseUrl)].concat(_toConsumableArray(middleware), [window.mupDevTools ? window.mupDevTools() : noopMiddleware]);
	var middlewareEnhancer = _redux.applyMiddleware.apply(undefined, _toConsumableArray(middlewareToApply));

	var enhancer = (0, _redux.compose)(middlewareEnhancer, clickTrackEnhancer, window.devToolsExtension ? window.devToolsExtension() : function (fn) {
		return fn;
	} // this must be last enhancer
	);
	return enhancer(_redux.createStore);
}
//# sourceMappingURL=index.js.map