'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});


/**
 * A module for middleware that would like to make external calls through `fetch`
 */
var parseQueryResponse = exports.parseQueryResponse = function parseQueryResponse(queries) {
	return function (proxyResponse) {
		if (!proxyResponse.responses) {
			throw new Error(JSON.stringify(proxyResponse)); // treat like an API error
		}
		var responses = proxyResponse.responses;

		if (queries.length !== responses.length) {
			throw new Error('Responses do not match requests');
		}

		return responses.reduce(function (categorized, response, i) {
			var targetArray = response.error ? categorized.errors : categorized.successes;
			targetArray.push({ response: response, query: queries[i] });
			return categorized;
		}, { successes: [], errors: [] });
	};
};
//# sourceMappingURL=fetchUtils.js.map