'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.epicIgnoreAction = exports.getServer = exports.parseCookieHeader = exports.middlewareDispatcher = exports.createFakeStore = exports.MOCK_LOGGER = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.testCreateStore = testCreateStore;

var _hapi = require('hapi');

var _hapi2 = _interopRequireDefault(_hapi);

var _toughCookie = require('tough-cookie');

var _toughCookie2 = _interopRequireDefault(_toughCookie);

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/of');

require('rxjs/add/operator/do');

var _reduxObservable = require('redux-observable');

var _app = require('meetup-web-mocks/lib/app');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var MOCK_LOGGER = exports.MOCK_LOGGER = {
	debug: jest.fn(),
	info: jest.fn(),
	warn: jest.fn(),
	error: jest.fn()
};

var createFakeStore = exports.createFakeStore = function createFakeStore(fakeData) {
	return {
		getState: function getState() {
			return fakeData;
		},
		dispatch: function dispatch() {},
		subscribe: function subscribe() {}
	};
};

var middlewareDispatcher = exports.middlewareDispatcher = function middlewareDispatcher(middleware) {
	return function (storeData, action) {
		var dispatched = null;
		var dispatch = middleware(createFakeStore(storeData))(function (actionAttempt) {
			return dispatched = actionAttempt;
		});
		dispatch(action);
		return dispatched;
	};
};

var parseCookieHeader = exports.parseCookieHeader = function parseCookieHeader(cookieHeader) {
	var cookies = cookieHeader instanceof Array ? cookieHeader.map(_toughCookie2.default.parse) : [_toughCookie2.default.parse(cookieHeader)];

	return cookies.reduce(function (acc, cookie) {
		return _extends({}, acc, _defineProperty({}, cookie.key, cookie.value));
	}, {});
};

var getServer = exports.getServer = function getServer() {
	var config = _extends({}, _config2.default, { supportedLangs: ['en-US'] });
	var server = new _hapi2.default.Server();
	server.connection({ port: 0 });
	server.app = {
		logger: MOCK_LOGGER
	};
	server.settings.app = config;
	server.plugins = {
		'api-proxy': {
			duotoneUrls: []
		}
	};

	// mock the anonAuthPlugin
	server.decorate('request', 'authorize', function (request) {
		return function () {
			return _Observable.Observable.of(request);
		};
	}, { apply: true });
	server.decorate('request', 'trackActivity', function () {
		return {};
	});
	server.decorate('request', 'getLangPrefixPath', function () {
		return '/';
	});
	server.decorate('request', 'getLanguage', function () {
		return 'en-US';
	});
	server.logger = function () {
		return MOCK_LOGGER;
	};
	server.ext('onPreHandler', function (request, reply) {
		request.plugins.tracking = {};
		reply.continue();
	});
	return server;
};

var epicIgnoreAction = exports.epicIgnoreAction = function epicIgnoreAction(epic) {
	var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _app.MOCK_MEANINGLESS_ACTION;
	var store = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : createFakeStore(_app.MOCK_APP_STATE);
	return function () {
		var spyable = {
			notCalled: function notCalled() {}
		};
		spyOn(spyable, 'notCalled');
		var action$ = _reduxObservable.ActionsObservable.of(action);
		return epic(action$, store).do(spyable.notCalled, null, expect(spyable.notCalled).not.toHaveBeenCalled()).toPromise();
	};
};

var IDENTITY_REDUCER = function IDENTITY_REDUCER(state) {
	return state;
};
function testCreateStore(createStoreFn) {
	it('creates a store with store functions', function () {
		var basicStore = createStoreFn(IDENTITY_REDUCER);
		expect(basicStore.getState).toEqual(jasmine.any(Function));
		expect(basicStore.dispatch).toEqual(jasmine.any(Function));
	});
	it('creates a store with supplied initialState', function (done) {
		var initialState = { foo: 'bar' };
		var basicStore = createStoreFn(IDENTITY_REDUCER, initialState);
		basicStore.subscribe(function () {
			expect(basicStore.getState()).toEqual(initialState);
			done();
		});
		basicStore.dispatch({ type: 'dummy' });
	});
}
//# sourceMappingURL=testUtils.js.map